/* This file can be best viewed with tab spacing = 2
* in vim ":set ts=2"
*************************************************************************************
*                        Program to draw a geometric figure.
*
* This program creates a frame containing a toolbar and a canvas.
* A rectangle can be drawn on the canvas between two points.
* The points on the canvas can be selected by mouse clicks.
* The toolbar contains a comboBox and an uneditable text field.
* The comboBox is used to set the colour of the rectangle.
* Currently the text field is unused.
*
* Extend/modify the program to add following features:
* 1. Display the current co-ordinate of mouse(on the canvas) in the text field present in toolBar.
* 2. Add one more comboBox to select current geometric shape amongst line, rectangle, and ellipse. 
* 3. Add a push button which randomly chooses the colour of the figure from color comboBox.
*
* Hints: Graphics.drawLine(), Graphics.drawOval(), Random.nextInt()
***************************************************************************************/

/* import required packages */
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Random;
import java.awt.geom.*;
import java.lang.reflect.Array;
/**
 * Class ToolBar 
 * drawingCanvas : Instance of the DrawingCanvas object
 */
class ToolBar extends JToolBar implements ActionListener {

	static final public Color[] color ={Color.white, Color.black, Color.blue, Color.red, Color.green, Color.yellow};
	static final private String[] COLORNAMES = {"White","Black","Blue","Red","Green","Yellow"};
	static final private String BOXCOLOR = "ccolor";
	static final private String SHAPE="dshape";
	static final private String[] shapes= {"Rectangle","Line","Ellipse"};
	DrawingCanvas drawingCanvas;	

	/**
	 * Constructor adds the widgets(controls) in the toolbar
	 */
	public ToolBar(){
    addWidgets();
	}//end of constructor

	/**
	 * canvas : canvas in use
	 */
	public void setDrawingCanvas(DrawingCanvas canvas){
		drawingCanvas = canvas;
	}//end of setDrawingCanvas

	/**
	 * This function adds widgets in the toolbar. 
	 */
	protected void addWidgets() {
				JComboBox comboBox = null;
				JTextField textField= null;
				JComboBox comboBox2=null;
				JButton pushbutton=null;	  
				//comboBox for colour
			  comboBox= makeComboBox(COLORNAMES,BOXCOLOR); 
//        add(comboBox);
			comboBox2=makeComboBox(shapes,SHAPE);
//	add(comboBox2);
				//textField
				textField = makeTextField(); 
  //      add(textField);
				pushbutton=new JButton("Color");
				pushbutton.addActionListener(this);
				pushbutton.setActionCommand("randomise");
	add(pushbutton);
    add(comboBox);
	add(comboBox2);
   add(textField);

	}//end of addWidgets()

	/**
	 * Create and initialize the comboBox
	 * text : Items in the comboBox
	 * actionCommand : text to set actionCommand of comboBOx 
	 * @JComboBox : crated comboBox 
	 */
	protected JComboBox makeComboBox(String[]text , String actionCommand){
        JComboBox comboBox = new JComboBox(text);
        comboBox.setActionCommand(actionCommand);
        comboBox.addActionListener(this);
        
				return comboBox;
	}//end of makeComboBox()

	/**
	 * Create and initialize the textField
	 */
	protected JTextField makeTextField(){
        JTextField textField = new JTextField("");
				textField.setEditable(false);

				return textField;
	}//end of makeTextField()

	/**
	 * e : action received 
	 */
	public void actionPerformed(ActionEvent e) {
				if(e.getSource()==getComponentAtIndex(0)){
//					int randcolor=((int)Math.random()*6);
					Random r=new Random();
					int length=Array.getLength(COLORNAMES);
//					System.out.println("array length is "+length);
					int randcolor=Math.abs(r.nextInt())%(length-1);
				//	randcolor=randcolor<6?randcolor:5;
					drawingCanvas.setColor(color[randcolor]);
				//	System.out.println(randcolor);
					((JComboBox)getComponentAtIndex(1)).setSelectedIndex(randcolor);
					return;
				}
				drawingCanvas.setColor(color[((JComboBox)getComponentAtIndex(1)).getSelectedIndex()]);
				drawingCanvas.setShape(((JComboBox)getComponentAtIndex(2)).getSelectedIndex());
				drawingCanvas.repaint();
	}//end of actionPerformed()

}//end of class ToolBar

/**
 * Canvas for drawing figures
 * p1, p2 : Stores two corner points of the rectangle to be drawn
 * color : store color in which rectangle is drawn
 */
class DrawingCanvas extends Canvas implements MouseListener, MouseMotionListener{

	Point p1,p2,temp;
	Color color;	
	int shape=0;
	JToolBar mytoolbar=null;
	/**
	 * Sets event handlers and initializes p1, p2 and color
	 */
	public DrawingCanvas(){
		addMouseMotionListener(this);
		addMouseListener(this);
		color= Color.white;
		p1 = new Point();
		p2 = new Point();
		temp = new Point(-1,-1);
	}//end of constructor
	public void setToolBar(JToolBar toolbar){
		mytoolbar=toolbar;
	}
	public void setColor(Color c){
		color = c;
	}//end of setColor()
	public void setShape(int shape){
		this.shape=shape;
	}
	public void mouseMoved(MouseEvent me) {
		int xloc=me.getX();
		int yloc=me.getY();
		((JTextField)mytoolbar.getComponentAtIndex(3)).setText(""+xloc+","+yloc);
	}//end of mouseMoved()

	/** 
	 * Stores coordinates of topLeft and bottomRight corners of the rectangle in Points p1 and p2.
	 */
	public void mouseClicked(MouseEvent me) {
		if(temp.x==-1){
			temp.setLocation(me.getX(),me.getY());
		}
	  else{
			p2.setLocation(me.getX(),me.getY());
			p1.setLocation(temp);
			repaint();
		}
	}//end of mouseClicked()
 
	public void mouseDragged(MouseEvent me) {
	}
 
	public void mouseEntered(MouseEvent me) {
	}
  
	public void mouseExited(MouseEvent me) {
	}
  
	public void mousePressed(MouseEvent me) {
	}
  
	public void mouseReleased(MouseEvent me) {
	}

	/** Method paint(): paints the surface of the canvas.
	 *	Draws a rectangle in color currently selected in comboBox in the toolbar.
	 *	Position of the rectangle is obtained from the points p1 and p2.
	 */
	public void paint(Graphics g){
			 Graphics2D g2=(Graphics2D)g;
			int x,y;
			x = (p1.x<p2.x)?p1.x:p2.x;
			y = (p1.y<p2.y)?p1.y:p2.y;
			
			temp.setLocation(-1,-1);

			g.setColor(color);
			if(shape==0)
				g.drawRect(x,y,Math.abs(p1.x-p2.x),Math.abs(p1.y-p2.y));
			else if(shape==1){
				g.drawLine(p1.x,p1.y,p2.x,p2.y);
			}
			else if(shape==2){
				//g2.draw(new Arc2D.Double(new Rectangle2D.Double(x,y,Math.abs(p1.x-p2.x),Math.abs(p1.y-p2.y)),0,360,Arc2D.OPEN));
				g.drawOval(x,y,Math.abs(p1.x-p2.x),Math.abs(p1.y-p2.y));
			} 
				
	
	}//end of paint()
 
}//end of class DrawingCanvas

/** Class: Q1
 * Extends JPanel to include a toolbar and a canvas into it
 * An instance of this class is set as content pane of main window of the application.
 */
public class Q1 extends JPanel{ 
	public Q1() {
		super(new BorderLayout());

		//Create the toolbar.
		ToolBar toolBar = new ToolBar();
		//Create the canvas.
		DrawingCanvas drawingCanvas = new DrawingCanvas();
		//Lay out the main panel.

		toolBar.setDrawingCanvas(drawingCanvas);
		toolBar.setFloatable(false);
		drawingCanvas.setToolBar(toolBar);
		// Size the panel
		setPreferredSize(new Dimension(450, 130));
	
		// Add the canvas and the toolbar
		add(drawingCanvas);
		add(toolBar, BorderLayout.PAGE_END);
	}

	 /**
     * Create the GUI and show it.  
		 * Creates a JFrame and adds an instance of Panel Q1 to its content pane.
     */
	private static void createAndShowGUI() {
        //Make sure we have nice window decorations.
        JFrame.setDefaultLookAndFeelDecorated(true);

        //Create and set up the window.
	 			JFrame frame = new JFrame("Q1");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        //Create and set up the drawing pane.
        Q1 drawingPane = new Q1();
        drawingPane.setOpaque(true); //content panes must be opaque
        frame.setContentPane(drawingPane);

        //Display the window.
        frame.pack();
        frame.setVisible(true);
    }//end of createAndShowGUI() 

    public static void main(String[] args) {
        //creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
    }//end of main()
}//end of class Q1
